package sun.awt.motif;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.awt.peer.ComponentPeer;
import sun.awt.ObjectPeer;
import sun.awt.ScreenUpdater;
import sun.awt.UpdateClient;

public abstract class MComponentPeer extends ObjectPeer implements ComponentPeer, UpdateClient {
   Component target;
   int pData;

   abstract void create(MComponentPeer var1);

   void create(MComponentPeer var1, Object var2) {
      this.create(var1);
   }

   native void disposeEvent(Event var1);

   native void pInitialize();

   native void pShow();

   native void pHide();

   native void pEnable();

   native void pDisable();

   native void pReshape(int var1, int var2, int var3, int var4);

   native void pAddRepaint(int var1, int var2, int var3, int var4);

   native void pTriggerRepaint();

   native void pDispose();

   void initialize() {
      if (this.target.isVisible()) {
         this.show();
      } else {
         this.hide();
      }

      Color var1;
      if ((var1 = this.target.getForeground()) != null) {
         this.setForeground(var1);
      }

      if ((var1 = this.target.getBackground()) != null) {
         this.setBackground(var1);
      }

      Font var2;
      if ((var2 = this.target.getFont()) != null) {
         this.setFont(var2);
      }

      if (!this.target.isEnabled()) {
         this.disable();
      }

      Rectangle var3 = this.target.bounds();
      this.reshape(var3.x, var3.y, var3.width, var3.height);
      this.pInitialize();
   }

   MComponentPeer(Component var1, Object var2) {
      this.target = var1;
      Container var3 = var1.getParent();
      this.create((MComponentPeer)(var3 != null ? ((Component)var3).getPeer() : null), var2);
      this.initialize();
   }

   MComponentPeer(Component var1) {
      this.target = var1;
      Container var2 = var1.getParent();
      this.create((MComponentPeer)(var2 != null ? ((Component)var2).getPeer() : null));
      this.initialize();
   }

   public native void setForeground(Color var1);

   public native void setBackground(Color var1);

   public native void setFont(Font var1);

   public ColorModel getColorModel() {
      return MToolkit.getStaticColorModel();
   }

   public void show() {
      this.pShow();
   }

   public void hide() {
      this.pHide();
   }

   public void enable() {
      this.pEnable();
   }

   public void disable() {
      this.pDisable();
   }

   public int updatePriority() {
      return 5;
   }

   public void updateClient(Object var1) {
      this.pTriggerRepaint();
   }

   public void repaint(long var1, int var3, int var4, int var5, int var6) {
      this.pAddRepaint(var3, var4, var5, var6);
      SecurityManager.enablePrivilege("UniversalThreadAccess");
      ScreenUpdater.updater.notify(this, var1);
   }

   public void paint(Graphics var1) {
      var1.setColor(this.target.getForeground());
      var1.setFont(this.target.getFont());
      this.target.paint(var1);
   }

   public void update(Graphics var1) {
      var1.setColor(this.target.getForeground());
      var1.setFont(this.target.getFont());
      this.target.update(var1);
   }

   public void print(Graphics var1) {
      Dimension var2 = this.target.size();
      var1.setColor(this.target.getForeground());
      var1.setFont(this.target.getFont());
      var1.drawRect(0, 0, var2.width - 1, var2.height - 1);
      this.target.print(var1);
   }

   public void reshape(int var1, int var2, int var3, int var4) {
      this.pReshape(var1, var2, var3, var4);
   }

   public native boolean handleEvent(Event var1);

   public Dimension minimumSize() {
      return this.target.size();
   }

   public Dimension preferredSize() {
      return this.minimumSize();
   }

   public Toolkit getToolkit() {
      return Toolkit.getDefaultToolkit();
   }

   public Graphics getGraphics() {
      X11Graphics var1 = new X11Graphics(this);
      ((Graphics)var1).setColor(this.target.getForeground());
      ((Graphics)var1).setFont(this.target.getFont());
      return var1;
   }

   public Image createImage(ImageProducer var1) {
      return new X11Image(var1);
   }

   public Image createImage(int var1, int var2) {
      return new X11OffScreenImage(this.target, var1, var2);
   }

   public boolean prepareImage(Image var1, int var2, int var3, ImageObserver var4) {
      return MToolkit.prepareScrImage(var1, var2, var3, var4);
   }

   public int checkImage(Image var1, int var2, int var3, ImageObserver var4) {
      return MToolkit.checkScrImage(var1, var2, var3, var4);
   }

   public FontMetrics getFontMetrics(Font var1) {
      return X11FontMetrics.getFontMetrics(var1);
   }

   public void dispose() {
      if (super.applet != null) {
         super.applet.removeEvents(this);
      }

      this.pDispose();
   }

   public native void requestFocus();

   public native void nextFocus();

   protected void gotFocus() {
      if (super.applet != null) {
         XEvent var1 = new XEvent(new Event(this.target, 1004, (Object)null), this);
         super.applet.sendEvent(var1);
      } else {
         this.target.postEvent(new Event(this.target, 1004, (Object)null));
      }
   }

   protected void lostFocus() {
      if (super.applet != null) {
         XEvent var1 = new XEvent(new Event(this.target, 1005, (Object)null), this);
         super.applet.sendEvent(var1);
      } else {
         this.target.postEvent(new Event(this.target, 1005, (Object)null));
      }
   }

   protected void handleActionKey(long var1, int var3, int var4, int var5, int var6, int var7) {
      Event var8 = this.setData(var3, new Event(this.target, var1, 403, var4, var5, var6, var7, (Object)null));
      if (super.applet != null) {
         XEvent var9 = new XEvent(var8, this);
         super.applet.sendEvent(var9);
      } else {
         this.target.postEvent(var8);
         this.disposeEvent(var8);
      }
   }

   protected void handleActionKeyRelease(long var1, int var3, int var4, int var5, int var6, int var7) {
      Event var8 = this.setData(var3, new Event(this.target, var1, 404, var4, var5, var6, var7, (Object)null));
      if (super.applet != null) {
         XEvent var9 = new XEvent(var8, this);
         super.applet.sendEvent(var9);
      } else {
         this.target.postEvent(var8);
         this.disposeEvent(var8);
      }
   }

   protected void handleKeyPress(long var1, int var3, int var4, int var5, int var6, int var7) {
      Event var8 = this.setData(var3, new Event(this.target, var1, 401, var4, var5, var6, var7, (Object)null));
      if (super.applet != null) {
         XEvent var9 = new XEvent(var8, this);
         super.applet.sendEvent(var9);
      } else {
         this.target.postEvent(var8);
         this.disposeEvent(var8);
      }
   }

   protected void handleKeyRelease(long var1, int var3, int var4, int var5, int var6, int var7) {
      Event var8 = this.setData(var3, new Event(this.target, var1, 402, var4, var5, var6, var7, (Object)null));
      if (super.applet != null) {
         XEvent var9 = new XEvent(var8, this);
         super.applet.sendEvent(var9);
      } else {
         this.target.postEvent(var8);
         this.disposeEvent(var8);
      }
   }

   protected void handleMouseEnter(long var1, int var3, int var4) {
      if (super.applet != null) {
         XEvent var5 = new XEvent(new Event(this.target, var1, 504, var3, var4, 0, 0, (Object)null), this);
         super.applet.sendEvent(var5);
      } else {
         this.target.postEvent(new Event(this.target, var1, 504, var3, var4, 0, 0, (Object)null));
      }
   }

   protected void handleMouseExit(long var1, int var3, int var4) {
      if (super.applet != null) {
         XEvent var5 = new XEvent(new Event(this.target, var1, 505, var3, var4, 0, 0, (Object)null), this);
         super.applet.sendEvent(var5);
      } else {
         this.target.postEvent(new Event(this.target, var1, 505, var3, var4, 0, 0, (Object)null));
      }
   }

   protected void handleMouseDown(long var1, int var3, int var4, int var5, int var6, int var7, int var8, int var9) {
      Event var10 = this.setData(var3, new Event(this.target, var1, 501, var4, var5, 0, var9, (Object)null));
      var10.clickCount = var8;
      if (super.applet != null) {
         XEvent var11 = new XEvent(var10, this);
         super.applet.sendEvent(var11);
      } else {
         this.target.postEvent(var10);
      }
   }

   protected void handleMouseUp(long var1, int var3, int var4, int var5, int var6, int var7, int var8) {
      Event var9 = this.setData(var3, new Event(this.target, var1, 502, var4, var5, 0, var8, (Object)null));
      if (super.applet != null) {
         XEvent var10 = new XEvent(var9, this);
         super.applet.sendEvent(var10);
      } else {
         this.target.postEvent(var9);
      }
   }

   protected void handleMouseMoved(long var1, int var3, int var4, int var5, int var6, int var7, int var8) {
      Event var9 = this.setData(var3, new Event(this.target, var1, 503, var4, var5, 0, var8, (Object)null));
      if (super.applet != null) {
         XEvent var10 = new XEvent(var9, this);
         super.applet.sendEvent(var10);
      } else {
         this.target.postEvent(var9);
      }
   }

   protected void handleMouseDrag(long var1, int var3, int var4, int var5, int var6, int var7, int var8) {
      Event var9 = this.setData(var3, new Event(this.target, var1, 506, var4, var5, 0, var8, (Object)null));
      if (super.applet != null) {
         XEvent var10 = new XEvent(var9, this);
         super.applet.sendEvent(var10);
      } else {
         this.target.postEvent(var9);
      }
   }

   native Event setData(int var1, Event var2);

   void handleExpose(int var1, int var2, int var3, int var4) {
      if (super.applet != null) {
         Rectangle var10 = new Rectangle(var1, var2, var3, var4);
         Event var6 = new Event(this.target, 202, var10);
         XEvent var7 = new XEvent(var6, this);
         super.applet.sendEvent(var7);
      } else {
         Graphics var5 = this.getGraphics();

         try {
            var5.clipRect(var1, var2, var3, var4);
            this.paint(var5);
         } finally {
            var5.dispose();
         }

      }
   }

   void handleRepaint(int var1, int var2, int var3, int var4) {
      if (super.applet != null) {
         Rectangle var10 = new Rectangle(var1, var2, var3, var4);
         Event var6 = new Event(this.target, 206, var10);
         XEvent var7 = new XEvent(var6, this);
         super.applet.sendEvent(var7);
      } else {
         Graphics var5 = this.getGraphics();

         try {
            var5.clipRect(var1, var2, var3, var4);
            this.update(var5);
         } finally {
            var5.dispose();
         }

      }
   }

   public String toString() {
      return this.getClass().getName() + "[" + this.target + "]";
   }
}
