package sun.awt.motif;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.ImageObserver;
import sun.awt.image.Image;
import sun.awt.image.ImageRepresentation;
import sun.awt.image.OffScreenImageSource;

public class X11Graphics extends Graphics {
   int pData;
   Color foreground;
   Font font;
   int originX;
   int originY;
   Image image;

   private native void createFromComponent(MComponentPeer var1);

   private native void createFromGraphics(X11Graphics var1);

   private native void imageCreate(ImageRepresentation var1);

   private native void pSetFont(Font var1);

   private native void pSetForeground(Color var1);

   X11Graphics(X11Graphics var1) {
      this.createFromGraphics(var1);
      this.font = var1.getFont();
      if (this.font == null) {
         this.font = new Font("Dialog", 0, 12);
      }

      this.pSetFont(this.font);
   }

   X11Graphics(MComponentPeer var1) {
      this.createFromComponent(var1);
      this.font = var1.target.getFont();
      if (this.font == null) {
         this.font = new Font("Dialog", 0, 12);
      }

      this.pSetFont(this.font);
   }

   public X11Graphics(X11Image var1) {
      OffScreenImageSource var2 = (OffScreenImageSource)((Image)var1).getSource();
      this.imageCreate(var2.getImageRep());
      this.image = var1;
      this.font = new Font("Dialog", 0, 12);
      this.pSetFont(this.font);
   }

   public Graphics create() {
      X11Graphics var1 = new X11Graphics(this);
      var1.foreground = this.foreground;
      var1.font = this.font;
      var1.originX = this.originX;
      var1.originY = this.originY;
      var1.image = this.image;
      return var1;
   }

   public void translate(int var1, int var2) {
      this.originX += var1;
      this.originY += var2;
   }

   public native void dispose();

   public void setFont(Font var1) {
      if (var1 != null && this.font != var1) {
         this.font = var1;
         this.pSetFont(var1);
      }

   }

   public Font getFont() {
      return this.font;
   }

   public FontMetrics getFontMetrics(Font var1) {
      return X11FontMetrics.getFontMetrics(var1);
   }

   public void setColor(Color var1) {
      if (var1 != null && var1 != this.foreground) {
         this.pSetForeground(var1);
         this.foreground = var1;
      }

   }

   public Color getColor() {
      return this.foreground;
   }

   public native void setPaintMode();

   public native void setXORMode(Color var1);

   public native Rectangle getClipRect();

   public native void clipRect(int var1, int var2, int var3, int var4);

   public native void clearRect(int var1, int var2, int var3, int var4);

   public native void fillRect(int var1, int var2, int var3, int var4);

   public native void drawRect(int var1, int var2, int var3, int var4);

   public native void drawString(String var1, int var2, int var3);

   public native void drawChars(char[] var1, int var2, int var3, int var4, int var5);

   public native void drawBytes(byte[] var1, int var2, int var3, int var4, int var5);

   public native int drawStringWidth(String var1, int var2, int var3);

   public native int drawCharsWidth(char[] var1, int var2, int var3, int var4, int var5);

   public native int drawBytesWidth(byte[] var1, int var2, int var3, int var4, int var5);

   public native void drawLine(int var1, int var2, int var3, int var4);

   public boolean drawImage(java.awt.Image var1, int var2, int var3, ImageObserver var4) {
      X11Image var5 = (X11Image)var1;
      if (((Image)var5).hasError()) {
         if (var4 != null) {
            var4.imageUpdate(var1, 192, -1, -1, -1, -1);
         }

         return false;
      } else {
         ImageRepresentation var6 = var5.getImageRep(-1, -1);
         return var6.drawImage(this, var2, var3, (Color)null, var4);
      }
   }

   public boolean drawImage(java.awt.Image var1, int var2, int var3, int var4, int var5, ImageObserver var6) {
      if (var4 != 0 && var5 != 0) {
         X11Image var7 = (X11Image)var1;
         if (((Image)var7).hasError()) {
            if (var6 != null) {
               var6.imageUpdate(var1, 192, -1, -1, -1, -1);
            }

            return false;
         } else {
            if (var4 < 0) {
               var4 = -1;
            }

            if (var5 < 0) {
               var5 = -1;
            }

            ImageRepresentation var8 = var7.getImageRep(var4, var5);
            return var8.drawImage(this, var2, var3, (Color)null, var6);
         }
      } else {
         return true;
      }
   }

   public boolean drawImage(java.awt.Image var1, int var2, int var3, Color var4, ImageObserver var5) {
      X11Image var6 = (X11Image)var1;
      if (((Image)var6).hasError()) {
         if (var5 != null) {
            var5.imageUpdate(var1, 192, -1, -1, -1, -1);
         }

         return false;
      } else {
         ImageRepresentation var7 = var6.getImageRep(-1, -1);
         return var7.drawImage(this, var2, var3, var4, var5);
      }
   }

   public boolean drawImage(java.awt.Image var1, int var2, int var3, int var4, int var5, Color var6, ImageObserver var7) {
      if (var4 != 0 && var5 != 0) {
         X11Image var8 = (X11Image)var1;
         if (((Image)var8).hasError()) {
            if (var7 != null) {
               var7.imageUpdate(var1, 192, -1, -1, -1, -1);
            }

            return false;
         } else {
            if (var4 < 0) {
               var4 = -1;
            }

            if (var5 < 0) {
               var5 = -1;
            }

            ImageRepresentation var9 = var8.getImageRep(var4, var5);
            return var9.drawImage(this, var2, var3, var6, var7);
         }
      } else {
         return true;
      }
   }

   public native void copyArea(int var1, int var2, int var3, int var4, int var5, int var6);

   public native void drawRoundRect(int var1, int var2, int var3, int var4, int var5, int var6);

   public native void fillRoundRect(int var1, int var2, int var3, int var4, int var5, int var6);

   public native void drawPolygon(int[] var1, int[] var2, int var3);

   public native void fillPolygon(int[] var1, int[] var2, int var3);

   public native void drawOval(int var1, int var2, int var3, int var4);

   public native void fillOval(int var1, int var2, int var3, int var4);

   public native void drawArc(int var1, int var2, int var3, int var4, int var5, int var6);

   public native void fillArc(int var1, int var2, int var3, int var4, int var5, int var6);

   public String toString() {
      return this.getClass().getName() + "[" + this.originX + "," + this.originY + "]";
   }
}
